@@ -3,17 +3,17 @@ from __future__ import annotations
 import asyncio
 import logging
 import socket
-from asyncio import Lock
+from asyncio import Transport
 from typing import Callable, Mapping, Optional
 
 import async_timeout
 
-from roborock.api import SPECIAL_COMMANDS, RoborockClient
-from roborock.containers import RoborockLocalDeviceInfo
-from roborock.exceptions import CommandVacuumError, RoborockConnectionException, RoborockException, RoborockTimeout
-from roborock.roborock_message import RoborockMessage, RoborockParser
-from roborock.typing import CommandInfoMap, RoborockCommand
-from roborock.util import get_running_loop_or_create_one
+from .api import SPECIAL_COMMANDS, RoborockClient, QUEUE_TIMEOUT
+from .containers import RoborockLocalDeviceInfo
+from .exceptions import CommandVacuumError, RoborockConnectionException, RoborockException
+from .roborock_message import RoborockMessage, RoborockParser
+from .typing import CommandInfoMap, RoborockCommand
+from .util import get_running_loop_or_create_one
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -30,15 +30,15 @@ class RoborockLocalClient(RoborockClient):
             )
             for device_id, device_info in devices_info.items()
         }
-        self._mutex = Lock()
         self._batch_structs: list[RoborockMessage] = []
         self._executing = False
 
-    async def async_connect(self):
+    async def async_connect(self) -> None:
         await asyncio.gather(*[listener.connect() for listener in self.device_listener.values()])
 
     async def async_disconnect(self) -> None:
-        await asyncio.gather(*[listener.disconnect() for listener in self.device_listener.values()])
+        for listener in self.device_listener.values():
+            listener.disconnect()
 
     def build_roborock_message(self, method: RoborockCommand, params: Optional[list] = None) -> RoborockMessage:
         secured = True if method in SPECIAL_COMMANDS else False
@@ -103,7 +103,7 @@ class RoborockSocket(socket.socket):
         return self._closed
 
 
-class RoborockSocketListener:
+class RoborockSocketListener(asyncio.Protocol):
     roborock_port = 58867
 
     def __init__(
@@ -111,64 +111,48 @@ class RoborockSocketListener:
         ip: str,
         local_key: str,
         on_message: Callable[[list[RoborockMessage]], None],
-        timeout: float | int = 4,
+        timeout: float | int = QUEUE_TIMEOUT,
     ):
         self.ip = ip
         self.local_key = local_key
-        self.socket = RoborockSocket(socket.AF_INET, socket.SOCK_STREAM)
-        self.socket.setblocking(False)
         self.loop = get_running_loop_or_create_one()
         self.on_message = on_message
         self.timeout = timeout
-        self.is_connected = False
-        self._mutex = Lock()
         self.remaining = b""
+        self.transport: Transport | None = None
 
-    async def _main_coro(self):
-        while not self.socket.is_closed:
-            try:
-                message = await self.loop.sock_recv(self.socket, 4096)
-                try:
-                    if self.remaining:
-                        message = self.remaining + message
-                        self.remaining = b""
-                    (parser_msg, remaining) = RoborockParser.decode(message, self.local_key)
-                    self.remaining = remaining
-                    self.on_message(parser_msg)
-                except Exception as e:
-                    _LOGGER.exception(e)
-            except BrokenPipeError as e:
-                _LOGGER.exception(e)
-                await self.disconnect()
+    def data_received(self, message):
+        if self.remaining:
+            message = self.remaining + message
+            self.remaining = b""
+        (parser_msg, remaining) = RoborockParser.decode(message, self.local_key)
+        self.remaining = remaining
+        self.on_message(parser_msg)
+
+    def connection_lost(self, exc):
+        print("The server closed the connection")
+
+    def is_connected(self):
+        return self.transport and self.transport.is_reading()
 
     async def connect(self):
-        async with self._mutex:
-            if not self.is_connected or self.socket.is_closed:
-                self.socket = RoborockSocket(socket.AF_INET, socket.SOCK_STREAM)
-                self.socket.setblocking(False)
-                try:
-                    async with async_timeout.timeout(self.timeout):
-                        _LOGGER.info(f"Connecting to {self.ip}")
-                        await self.loop.sock_connect(self.socket, (self.ip, 58867))
-                        self.is_connected = True
-                except Exception as e:
-                    await self.disconnect()
-                    raise RoborockConnectionException(f"Failed connecting to {self.ip}") from e
-                self.loop.create_task(self._main_coro())
+        try:
+            if not self.is_connected():
+                async with async_timeout.timeout(self.timeout):
+                    _LOGGER.info(f"Connecting to {self.ip}")
+                    self.transport, _ = await self.loop.create_connection(lambda: self, self.ip, 58867)
+        except Exception as e:
+            raise RoborockConnectionException(f"Failed connecting to {self.ip}") from e
 
-    async def disconnect(self):
-        self.socket.close()
-        self.is_connected = False
+    def disconnect(self):
+        if self.transport:
+            self.transport.close()
 
     async def send_message(self, data: bytes) -> None:
         await self.connect()
         try:
-            async with self._mutex:
-                async with async_timeout.timeout(self.timeout):
-                    await self.loop.sock_sendall(self.socket, data)
-        except (asyncio.TimeoutError, asyncio.CancelledError):
-            await self.disconnect()
-            raise RoborockTimeout(f"Timeout after {self.timeout} seconds waiting for response") from None
-        except BrokenPipeError as e:
-            await self.disconnect()
+            if not self.transport:
+                raise RoborockException("Can not send message without connection")
+            self.transport.write(data)
+        except Exception as e:
             raise RoborockException(e) from e
